import java.util.Vector;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.bluetooth.*;
import javax.microedition.io.*;

public class ChatController extends MIDlet implements CommandListener {
    private Display display;
    private Form mainForm;
    private ChoiceGroup devices;
    private TextField inTxt;
    private TextField outTxt;
    private Command exit;
    private Command start;
    private Command connect;
    private Command send;
    private Command select;
    private StringItem status;

    private LocalDevice local;
    private DiscoveryAgent agent;
    private StreamConnectionNotifier notifier;
    private StreamConnection conn = null;
    private InputStream input = null;
    private OutputStream output = null;
    private RemoteDevice[] rDevices = null;

    private static final String UUID_STRING = "112233445566778899AABBCCDDEEFF";

    private boolean running = false;

    public ChatController() {
        devices = new ChoiceGroup(null,Choice.EXCLUSIVE);
        inTxt = new TextField("incoming msg:","",256,TextField.ANY);
        outTxt = new TextField("outgoing msg:","",256,TextField.ANY);
        exit = new Command("EXIT",Command.EXIT,1);
        start = new Command("START",Command.SCREEN,2);
        connect = new Command("CONNECT",Command.SCREEN,2);
        send = new Command("SEND",Command.SCREEN,2);
        select = new Command("SELECT",Command.SCREEN,2);
        status = new StringItem("status : ",null);
		
		mainForm = new Form("CHAT");
        mainForm.append(status);
        mainForm.addCommand(exit);
        mainForm.setCommandListener(this);
		
        display = Display.getDisplay(this);
    }

    protected void startApp() {
        running = true;
        mainForm.addCommand(start);
        mainForm.addCommand(connect);
        display.setCurrent(mainForm);

        try {
            local = LocalDevice.getLocalDevice();
            agent = local.getDiscoveryAgent();
        } catch(BluetoothStateException ex) {
            status.setText("BluetoothStateException unable to start:" + ex.getMessage());
            try {
                Thread.sleep(1000);
	    } catch(InterruptedException e) {}
            notifyDestroyed();
        }
    }

    protected void pauseApp() {
        running = false;
		//releaseResources();
    }

    protected void destroyApp(boolean uncond) {
        running = false;
		releaseResources();
        notifyDestroyed();
    }

    public void commandAction(Command cmd, Displayable disp) {
        if(cmd==exit) destroyApp(true);
        else if(cmd==start) {
            new Thread() {
				public void run() {
                    startServer();
				}
            }.start();
        }
        else if(cmd==connect) {
            status.setText("searching for devices...");
            mainForm.removeCommand(connect);
            mainForm.removeCommand(start);
            mainForm.append(devices);

			//search for other Bluetooth devices
            DeviceDiscoverer discoverer = new DeviceDiscoverer(this);
            try {
                agent.startInquiry(DiscoveryAgent.GIAC,discoverer);
            } catch(IllegalArgumentException ex) {
                status.setText("BluetoothStateException :" + ex.getMessage());
            } catch(NullPointerException ex) {
                status.setText("BluetoothStateException :" + ex.getMessage());
            } catch(BluetoothStateException ex) {
                status.setText("BluetoothStateException :" + ex.getMessage());
            }
        }
        else if(cmd==select) {
            status.setText("searching devices for service...");
            int index = devices.getSelectedIndex();
            mainForm.delete(mainForm.size()-1); //deletes choiceGroup
            mainForm.removeCommand(select);

			//search for the MobileChat application on a given device
            ServiceDiscoverer serviceDListener = new ServiceDiscoverer(this);
            int attrSet[] = {0x0100}; //returns service name attribute
            UUID[] uuidSet = {new UUID(UUID_STRING,false)};
            try {
				agent.searchServices(attrSet,uuidSet,rDevices[index],serviceDListener);
            } catch(IllegalArgumentException ex) {
                status.setText("BluetoothStateException :" + ex.getMessage());
            } catch(NullPointerException ex) {
                status.setText("BluetoothStateException :" + ex.getMessage());
            } catch(BluetoothStateException ex) {
                status.setText("BluetoothStateException :" + ex.getMessage());
            }
        } 
        else if(cmd==send) {
            new Thread() {
                public void run() {
                    sendMessage();
				}
            }.start();
		}
    }

    //this method is called from DeviceDiscoverer when device inquiry finishes
    public void deviceInquiryFinished(RemoteDevice[] rDevices,String message) {
		status.setText(message);
        devices.deleteAll();
        this.rDevices = rDevices;

		for(int i=0; i<rDevices.length; i++) {
            String name = "";
            try {
				//get the device name
                name = rDevices[i].getFriendlyName(false);
			} catch(IOException ioe) {
				name = rDevices[i].getBluetoothAddress();
            }

            devices.append(name,null);
		}

		if (devices.size() == 0) status.setText("No devices found!");
        else mainForm.addCommand(select);
    }

    //called by ServiceDiscoverer when service search gets completed
    public void serviceSearchFinished(ServiceRecord service,String message) {
		status.setText(message);
		String url = "";

		try {
			//retrieve the connection url from the service's ServiceRecord
            url = service.getConnectionURL(ServiceRecord.NOAUTHENTICATE_NOENCRYPT,false);
			
			//open the connection streams
            conn = (StreamConnection)Connector.open(url);
            input = conn.openInputStream();
            output = conn.openOutputStream();
            status.setText("connected...");

            new Thread() {
				public void run() {
                    startReciever();
				}
            }.start();

        } catch (NullPointerException ex) {
            status.setText("Couldn't find service!");
        }catch (SecurityException ex) {
            status.setText("Connection not authorized!");
        } catch (IllegalArgumentException ex) {
        } catch(IOException ex) {
            status.setText("IOException :" + ex.getMessage());
        }
    }

    // this method starts connection chat server from server mode
    public void startServer() {
        status.setText("server starting...");
		mainForm.removeCommand(connect);
		mainForm.removeCommand(start);

		try {
            local.setDiscoverable(DiscoveryAgent.GIAC);
            notifier = (StreamConnectionNotifier)Connector.open("btspp://localhost:"+UUID_STRING+";name=MyMobileChat");
            status.setText("server running...");

			//start waiting for the clients
            conn = notifier.acceptAndOpen();
            input = conn.openInputStream();
            output = conn.openOutputStream();

            new Thread() {
                public void run() {
                    startReciever();
                }
            }.start();
        } catch(IOException ex) {
            status.setText("IOException :" + ex.getMessage());
		}
    }

    //starts a message reciever listening for incomming message
    public void startReciever() {
        mainForm.addCommand(send);
        mainForm.append(inTxt);
		mainForm.append(outTxt);

		while(running && input != null) {
            try {
                if(input.available() != 0) {
                    byte[] data = new byte[input.available()];
                    input.read(data);
                    String message = new String(data);
                    inTxt.setString(message);
                }
			} catch(IOException ex) {
				status.setText("IOException :" + ex.getMessage());
            }
        }
    }

    //sends a message over connection
    public void sendMessage() {
        try {
            String message = outTxt.getString();
            byte[] data = message.getBytes();
            output.write(data);
            output.flush();
		} catch (IOException ex) {
            status.setText("IOException :" + ex.getMessage());
		}
    }

    public void releaseResources() {
        try {
            if (input != null) input.close();
            if (output != null) output.close();
            if (conn != null) conn.close();
			if(notifier != null) notifier.close();
		} catch(IOException ex) {
			status.setText("IOException :" + ex.getMessage());
		}
    }
}
